@file:OptIn(DelicateCoroutinesApi::class)

package com.bing89.travebing

import android.Manifest.permission
import android.annotation.SuppressLint
import android.content.ContentValues
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.location.Location
import android.location.LocationListener
import android.location.LocationManager
import android.net.Uri
import android.os.Bundle
import android.provider.MediaStore
import android.util.Log
import android.widget.Button
import android.widget.EditText
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import androidx.annotation.RequiresPermission
import androidx.appcompat.app.AppCompatActivity
import androidx.camera.core.CameraSelector
import androidx.camera.core.ImageCapture
import androidx.camera.core.ImageCaptureException
import androidx.camera.lifecycle.ProcessCameraProvider
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import com.bing89.travebing.R.id
import com.bing89.travebing.R.layout
import kotlinx.coroutines.DelicateCoroutinesApi
import okhttp3.Call
import okhttp3.Callback
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.MultipartBody
import okhttp3.OkHttpClient
import okhttp3.Request
import okhttp3.RequestBody.Companion.asRequestBody
import okhttp3.Response
import java.io.BufferedInputStream
import java.io.BufferedOutputStream
import java.io.File
import java.io.FileInputStream
import java.io.FileOutputStream
import java.io.IOException
import java.io.InputStream
import java.text.SimpleDateFormat
import java.util.Locale
import java.util.Timer
import java.util.TimerTask


class MainActivity : AppCompatActivity() {

    private lateinit var startButton: Button
    private lateinit var previewImageView: ImageView
    private lateinit var gpsInfoTextView: TextView
    private var imageCapture: ImageCapture? = null
    private lateinit var planText: EditText
    private var isMonitoring = false
    private var photoFilePath: String? = null
    private var locationManager: LocationManager? = null
    private lateinit var lat: String
    private lateinit var lon: String
    private lateinit var currentImg:String
    private  lateinit var context:Context
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        lat = ""
        lon = ""
        currentImg = ""
        context = baseContext
        setContentView(layout.activity_main)
        startButton = findViewById(id.startButton)
        previewImageView = findViewById(id.previewImageView)
        gpsInfoTextView = findViewById(id.gpsInfoTextView)
        planText = findViewById(id.planText)
        startCamera()
        startButton.setOnClickListener {
            if (isMonitoring) {
                stopMonitoring()
                planText.isActivated = true
            } else {
                planText.isActivated = false
                startMonitoring()
            }
        }
        checkPermissions()
        getLocation()
        locationManager = getSystemService(Context.LOCATION_SERVICE) as LocationManager
    }

    private  fun updateWayCallback(file: File):Callback{
       return object:Callback{
            override fun onFailure(call: Call, e: IOException) {
                file.delete()
                println("Fail")
            }

            override fun onResponse(call: Call, response: Response) {
                println(response.body.toString())
                file.delete()
            }

        }
    }
    private  fun updateWay(img:InputStream){
        Log.d(TAG, "$img.jpg")
        val client = OkHttpClient()

        var builder: MultipartBody.Builder =  MultipartBody.Builder()
            .setType(MultipartBody.FORM)
            .addFormDataPart("lat", lat)
            .addFormDataPart("lon", lon)
        val dst = File("${context.externalCacheDir}/tmp")
        copyfile(img, dst )
        val requestFile = dst.asRequestBody(MEDIA_TYPE_FILE)
        builder = builder.addFormDataPart("file", "1.jpg", requestFile)
        val multipartBody: MultipartBody = builder.build()
        val request = Request.Builder()
            .url("http://travebing.bing89.com/api/v1/plans/" + planText.text + "/ways")
            .post(multipartBody)
            .build()

        val call: Call = client.newCall(request)
        call.enqueue(updateWayCallback(dst))
    }
    private fun startMonitoring() {
        isMonitoring = true
        startButton.text = "休息"
        val timer = Timer()
        val task = object : TimerTask() {
            override fun run() {
                // 每隔 1 秒钟执行一次此方法
                // 在这里写下你需要执行的代码
               takePhoto()
            }
        }

        timer.schedule(task, 0, 300000) // 每隔 300 秒钟执行一次

//        updateWay(img)
    }

    private fun stopMonitoring() {
        isMonitoring = false
        startButton.text = "出发"
    }

    @RequiresPermission(
        anyOf = [permission.ACCESS_COARSE_LOCATION,
            permission.ACCESS_FINE_LOCATION,
            permission.INTERNET,
            permission.ACCESS_MEDIA_LOCATION,
            permission.ACCESS_BACKGROUND_LOCATION,
            permission.ACCESS_NETWORK_STATE,
            permission.CAMERA,
            permission.WRITE_EXTERNAL_STORAGE,
            permission.READ_EXTERNAL_STORAGE,
            permission.ACCESS_FINE_LOCATION],
    )

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (resultCode == PHOTO_REQUEST_CODE) {
//            doSomeOperations()
        }
    }

    private fun checkPermissions(): Boolean {
        val requiredPermissions = arrayOf(
            permission.ACCESS_COARSE_LOCATION,
            permission.ACCESS_FINE_LOCATION,
            permission.INTERNET,
            permission.ACCESS_MEDIA_LOCATION,
            permission.ACCESS_BACKGROUND_LOCATION,
            permission.ACCESS_NETWORK_STATE,
            permission.CAMERA,
            permission.WRITE_EXTERNAL_STORAGE,
            permission.READ_EXTERNAL_STORAGE,
            permission.ACCESS_FINE_LOCATION
        )

        for (permission in requiredPermissions) {
            if (ActivityCompat.checkSelfPermission(
                    this,
                    permission
                ) != PackageManager.PERMISSION_GRANTED
            ) {
                ActivityCompat.requestPermissions(
                    this,
                    requiredPermissions,
                    PERMISSION_REQUEST_CODE
                )
                return false
            }
        }

        return true
    }

    //define the listener
    private val locationListener: LocationListener = object : LocationListener {
        override fun onLocationChanged(location: Location) {
            lat = location.latitude.toString()
            lon = location.longitude.toString()
            updateGpsInfo(location)
        }

        override fun onStatusChanged(provider: String, status: Int, extras: Bundle) {}
        override fun onProviderEnabled(provider: String) {}
        override fun onProviderDisabled(provider: String) {}
    }

    private fun getLocation() {
        val locMan = getSystemService(Context.LOCATION_SERVICE) as LocationManager
        val checkCameraPermission =
            ContextCompat.checkSelfPermission(this, permission.ACCESS_FINE_LOCATION)
        val checkCallPhonePermission =
            ContextCompat.checkSelfPermission(this, permission.ACCESS_COARSE_LOCATION)
        if (checkCallPhonePermission != PackageManager.PERMISSION_GRANTED || checkCameraPermission != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(
                this, arrayOf(
                    permission.ACCESS_COARSE_LOCATION,
                    permission.ACCESS_FINE_LOCATION
                ), 22
            )
        }
        try {
            // Request location updates
            val providers = locMan.getProviders(false)
            var provider = LocationManager.NETWORK_PROVIDER
            if (providers.size != 0) {
                if (providers.contains(LocationManager.NETWORK_PROVIDER)) {
                    provider = LocationManager.NETWORK_PROVIDER
                }else if(providers.contains(LocationManager.GPS_PROVIDER)){
                    provider = LocationManager.GPS_PROVIDER
                } else if (providers.contains(LocationManager.PASSIVE_PROVIDER)) {
                    provider = LocationManager.PASSIVE_PROVIDER
                }
            }
            Toast.makeText(baseContext, "使用" + provider + "定位", Toast.LENGTH_SHORT).show()
            Log.d(TAG, "==================>>>> get location from gps")
            locMan.requestLocationUpdates(provider, 10000L, 100f, locationListener)
        } catch (ex: SecurityException) {
            Log.d(TAG, "Security Exception, no location available")
        }
    }


    private fun updateGpsInfo(location: Location) {
        val latitude = location.latitude
        val longitude = location.longitude
        val gpsInfo = "Latitude: $latitude, Longitude: $longitude"
        gpsInfoTextView.text = gpsInfo
    }

    companion object {
        private const val PERMISSION_REQUEST_CODE = 1001
        private const val PHOTO_REQUEST_CODE = 1002
        private const val TAG = "Travebing"
        private const val FILENAME_FORMAT = "yyyy-MM-dd-HH-mm-ss-SSS"
        private const val REQUEST_CODE_PERMISSIONS = 10
        val MEDIA_TYPE_JSON = "application/json; charset=utf-8".toMediaType()
        val MEDIA_TYPE_MULTIPART = "multipart/form-data".toMediaType()
        val MEDIA_TYPE_FILE = "application/octet-stream".toMediaType()
    }

    private fun startCamera() {
        val cameraProviderFuture = ProcessCameraProvider.getInstance(this)

        cameraProviderFuture.addListener({
            val cameraProvider: ProcessCameraProvider = cameraProviderFuture.get()
            imageCapture = ImageCapture.Builder().build()
            // Select back camera as a default
            val cameraSelector = CameraSelector.DEFAULT_BACK_CAMERA

            try {
                // Unbind use cases before rebinding
                cameraProvider.unbindAll()

                // Bind use cases to camera
                cameraProvider.bindToLifecycle(
                    this, cameraSelector, imageCapture
                )

            } catch (exc: Exception) {
                Log.e(TAG, "Use case binding failed", exc)
            }

        }, ContextCompat.getMainExecutor(this))
    }
    /**
     * 单个文件复制
     */
    fun copyfile(srcFile: InputStream, destFile: File) {
        val fos = FileOutputStream(destFile)
        srcFile.copyTo(fos)
        fos.close()
        srcFile.close()
    }

    private fun takePhoto(): String {
        // Get a stable reference of the modifiable image capture use case
        val imageCapture = imageCapture ?: return ""
        Log.d(TAG, "================>>>>> now do capture photo")
        // Create time stamped name and MediaStore entry.
        val name = SimpleDateFormat(FILENAME_FORMAT, Locale.US)
            .format(System.currentTimeMillis())
        val contentValues = ContentValues().apply {
            put(MediaStore.MediaColumns.DISPLAY_NAME, name)
            put(MediaStore.MediaColumns.MIME_TYPE, "image/jpeg")
            put(MediaStore.Images.Media.RELATIVE_PATH, "Pictures/travebing")
        }

        // Create output options object which contains file + metadata
        val outputOptions = ImageCapture.OutputFileOptions
            .Builder(
                contentResolver,
                MediaStore.Images.Media.EXTERNAL_CONTENT_URI,
                contentValues
            )
            .build()

        // Set up image capture listener, which is triggered after photo has
        // been taken
        imageCapture.takePicture(
            outputOptions,
            ContextCompat.getMainExecutor(this),
            object : ImageCapture.OnImageSavedCallback {
                override fun onError(exc: ImageCaptureException) {
                    Log.e(TAG, "+++++++++++Photo capture failed: ${exc.message}", exc)
                }
                @SuppressLint("Recycle")
                override fun onImageSaved(output: ImageCapture.OutputFileResults) {
                    previewImageView.setImageURI(output.savedUri)
                    val msg = "--------- Photo capture succeeded: ${output.savedUri}"
                    Toast.makeText(baseContext, msg, Toast.LENGTH_SHORT).show()
                    Log.d(TAG, msg)
                    val resolve = context.contentResolver
                    val uri = Uri.withAppendedPath(output.savedUri, "")
                    val ins:InputStream = resolve.openInputStream(uri)!!
                    updateWay(ins)
                }
            }
        )
        return name
    }

}
